# Django uWSGI Nginx server without venv installation guide
- Example project name: savessh
- Example user name: keju

### Prerequisites
- Server connected to the internet
- Raspberry Pi OS Lite or Ubuntu Server as the operating system

### Server setup
- Install all available updates (`sudo apt update && sudo apt upgrade`)
- Check if Python is installed correctly (`python`)
    * If not, install it (`sudo apt install python`) and check again
- Check if pip is installed correctly (`pip`)
    * If not, install it (`sudo apt install pip`) and check again

### Install required Python packages
- Install Django using pip (`pip install django --break-system-packages`)
- Install uWSGI using pip (`pip install uwsgi --break-system-packages`)
- Check if Django and uWSGI work (`django-admin`), (`uwsgi`)
    * If not, move the installed binaries manually into the `/bin` folder 
        ```bash
        cd /home/keju/.local/bin/
        sudo mv django-admin /bin/django-admin
        sudo mv uwsgi /bin/uwsgi
        ```

### Create and test a Django project
- Create a prject (`django-admin startproject savessh`)
    * Navigate into the newly created project folder (`cd savessh`)
- Test the project (`python manage.py runserver 192.168.178.100:8000`) (Replace the IP adress with the one of your server, leave port 8000)
    * Navigate to the IP address with port 8000 in your browser and you should see "DisallowedHost at /"
    * Fix this by opening the savessh/settings.py file and adding your IP address to ALLOWED_HOSTS like this: `ALLOWED_HOSTS = ['192.168.178.100']`
    * Open the IP address in your browser again and you should see the default Django landing page

### Get started with uWSGI
- Create a file called `test.py` in your user folder, not the project folder, with the following content:
    ```python
    def application(env, start_response):
        start_response('200 OK', [('Content-Type','text/html')])
        return [b"Hello World!"]
    ```
- Test uWSGI (`uwsgi --http :8000 --wsgi-file test.py`)
    * Visit the IP address with port 8000 again in a browser and you should see the text "Hello World"
- Test uWSGI with Django (`uwsgi --http :8000 --module savessh.wsgi`)
    * Visit the IP address with port 8000 again in a browser and you should see the default Django landing page

### Configure Nginx
- Install Nginx (`sudo apt install nginx`)
- Create a configuration file for Nginx (`sudo nano /etc/nginx/sites-available/savessh.conf`)
    * Add the following content and change things to your needs
        ```bash
        # The upstream component nginx needs to connect to
        upstream django {
            server unix:///home/keju/savessh/savessh.sock;
        }
        # Configuration of the server
        server {
            listen      80;
            server_name savessh.com;
            charset     utf-8;
            # max upload size
            client_max_body_size 75M;
            # Django media and static files
            location /media  {
                alias /home/keju/savessh/media;
            }
            location /static {
                alias /home/keju/savessh/static;
            }
            # Send all non-media requests to the Django server
            location / {
                uwsgi_pass  django;
                include     /home/keju/savessh/uwsgi_params;
            }
        }
        ```
- Edit a configuration file for Nginx (`sudo nano /etc/nginx/nginx.conf`)
    * Change `user www-data;` to your username, in this case `user keju;` and
- Create the `uwsgi_params` file (`sudo nano /home/keju/savessh/uwsgi_params`)
    * Add the following content
        ```ini
        uwsgi_param  QUERY_STRING       $query_string;
        uwsgi_param  REQUEST_METHOD     $request_method;
        uwsgi_param  CONTENT_TYPE       $content_type;
        uwsgi_param  CONTENT_LENGTH     $content_length;
        uwsgi_param  REQUEST_URI        $request_uri;
        uwsgi_param  PATH_INFO          $document_uri;
        uwsgi_param  DOCUMENT_ROOT      $document_root;
        uwsgi_param  SERVER_PROTOCOL    $server_protocol;
        uwsgi_param  REQUEST_SCHEME     $scheme;
        uwsgi_param  HTTPS              $https if_not_empty;
        uwsgi_param  REMOTE_ADDR        $remote_addr;
        uwsgi_param  REMOTE_PORT        $remote_port;
        uwsgi_param  SERVER_PORT        $server_port;
        uwsgi_param  SERVER_NAME        $server_name;
        ```
- Create a symbolic link for Nginx from `sites-available` to `sites-enabled` (`sudo ln -s /etc/nginx/sites-available/savessh.conf /etc/nginx/sites-enabled/`)
- Edit the savessh/settings.py file
    * Add `import os` at the very beginning before `from pythlib import Path`
    * Add `STATIC_ROOT = os.path.join(BASE_DIR, "static/")` at the very end after `STATIC_URL = '/static/'`
- Collect all the static files in one place (`python manage.py collectstatic`)
- Restart Nginx to apply all changes (`sudo systemctl restart nginx.service`)
- Save an image file (for example, your websites favicon) to the static folder in your Django project folder
- Visit the IP address with port 80 in your browser and add `/static/favicon.ico` (or the name of your file in the static folder) (`http://192.168.178.100:80/static/favicon.ico`) and you should see that image

### Get Nginx, uWSGI and Django to work together
- Run this command `uwsgi --socket savessh.sock --module savessh.wsgi` and visit your website again with port 80, you should see the default Django landing page
- **NOTE: nginx must be running for this to work**
    * If this does not work for you, try running `uwsgi --socket savessh.sock --module savessh.wsgi --chmod-socket=664`
    * If that still does not work try running `uwsgi --socket savessh.sock --module savessh.wsgi --chmod-socket=666`

### Configure uWSGI for production
- Create a configuration file at the root of your Django project (`sudo nano savessh_uwsgi.ini`) and add the following content
    ```ini
    [uwsgi]
    # Full path to Django project's root directory
    chdir            = /home/keju/savessh/
    # Django's WSGI file
    module           = savessh.wsgi
    # Full path to Python virtual env
    #home             = /home/keju/env/md
    # Enable uWSGI master process
    master          = true
    # Maximum number of worker processes
    processes       = 10
    # The socket (use the full path to be safe)
    socket          = /home/keju/savessh/savessh.sock
    # Socket permissions
    #chmod-socket    = 666
    # Clear environment on exit
    #vacuum          = true
    # Daemonize uWSGI and write messages into given log
    daemonize       = /home/keju/uwsgi-emperor.log
    ```
- Start uWSGI and link the ini file `uwsgi --ini savessh_uwsgi.ini`
- Run uWSGI in emperor mode. This will monitor the uWSGI config file directory for changes and will spawn vassals (i.e. instances) for each one it finds.
    ```bash
    cd /home/keju/
    mkdir vassals
    sudo ln -s /home/keju/savessh/savessh_uwsgi.ini /home/keju/vassals/
    ```
- Now run uWSGI in emperor mode as a test `uwsgi --emperor /home/keju/vassals --uid www-data --gid www-data`
- Finally, to make uWSGI start at boot, create a systemd file `sudo nano /etc/systemd/system/emperor.uwsgi.service`
    ```ini
    [Unit]
    Description=uwsgi emperor for savessh website
    After=network.target
    [Service]
    User=keju
    Restart=always
    ExecStart=/bin/uwsgi --emperor /home/keju/vassals --uid www-data --gid www-data
    [Install]
    WantedBy=multi-user.target
    ```
- Allow the service to start at boot `sudo systemctl enable emperor.uwsgi.service`
- Start the service without rebooting `sudo systemctl start emperor.uwsgi.service`
- When visisting your website on port 80 you should see it
- You can see the status of uWSGI with `sudo systemctl status emperor.uwsgi.service`
- You can stop uWSGI with `sudo systemctl stop emperor.uwsgi.service` 

# Install a SSL certificate
- Install snapd `sudo apt install snapd`
- Install certbot `sudo snap install --classic certbot`
- Create a required symbolic link `sudo ln -s /snap/bin/certbot /usr/bin/certbot`
- Get SSL certificate `sudo certbot --nginx` and follow the instructions
- **Dont forget to forward port 443 to be able to use HTTPS**

# Run Django channels in production with SSL
- Install redis and channels-redis
    ```bash
    pip install channels-redis --break-system-packages
    sudo apt install redis
    ```
- Test if redis is working correctly
    ```bash
    redis-cli
    ping
    ```
    - When executing `ping` in the `redis-cli` you should get a `PONG` as the response
- Replace
    ```python
    CHANNEL_LAYERS = {
        'default': {
            'BACKEND': 'channels.layers.InMemoryChannelLayer',
        },
    }
    ```
    with
    ```python
    CHANNEL_LAYERS = {
        "default": {
            "BACKEND": "channels_redis.core.RedisChannelLayer",
            "CONFIG": {
                "hosts": [("localhost", 6379)],
            },
        },
    }
    ```
    in `settings.py`
- Change your websocket connection in JavaScript to a secured websocket connection
    ```javascript
    'wss://'+ window.location.host + '/ws/chat/'
    ```
- Add this to your nginx sites-available savessh.conf (`sudo nano /etc/nginx/sites-available/savessh.conf`)
    ```bash
    # WebSocket handling
    location /ws/ {
        proxy_http_version 1.1;
        proxy_set_header Upgrade $http_upgrade;
        proxy_set_header Connection 'upgrade';
        proxy_redirect off;
        proxy_pass http://127.0.0.1:8001/ws/;
    }
    ```
- Run the Daphne server (`daphne -p 8001 savessh.asgi:application`)